"""Core pep438 utility functions"""
from __future__ import unicode_literals

import requests
try:
    import xmlrpclib
except:
    import xmlrpc.client as xmlprclib
import lxml.html
from requirements import parse


def valid_package(package_name):
    """Return bool if package_name is a valid package on PyPI"""
    response = requests.head('https://pypi.python.org/pypi/%s' % package_name)
    if response.status_code != 404:
        response.raise_for_status()
    return response.status_code != 404


def get_urls(package_name):
    """Return list of URLs on package's PyPI page that would be crawled"""
    response = requests.get('https://pypi.python.org/simple/%s' % package_name)
    response.raise_for_status()
    page = lxml.html.fromstring(response.content)
    crawled_urls = {link.get('href') for link in page.xpath('//a')
                    if link.get('rel') in ("homepage", "download")}
    return crawled_urls


def get_pypi_packages(fileobj):
    """Return all PyPI-hosted packages from file-like object"""
    return [p['name'] for p in parse(fileobj) if not p.get('uri')]


def get_pypi_user_packages(user):
    client = xmlrpclib.ServerProxy('https://pypi.python.org/pypi')
    return [x[1] for x in client.user_packages(user)]



"""Command-line interface for pep438 command"""
from __future__ import print_function, unicode_literals
import sys
from getopt import getopt, GetoptError

from clint import piped_in
from clint.textui import puts, columns, indent
from clint.textui.core import STDOUT, STDERR
from clint.textui.colored import green, red, blue

from . import __version__
from .core import (get_urls, get_pypi_packages, valid_package,
                   get_pypi_user_packages)


def version():
    print("pep438 version %s" % __version__)


def usage(error=False):
    """Print usage information and if error is ``True`` print to stderr"""
    out = STDERR if error else STDOUT
    options = [
        ("-h, --help", "Print this help message"),
        ("-v, --version", "Display version information"),
        ("-e, --errors-only", "Display errors only"),
    ]

    puts("\nUsage:")
    with indent(4):
        puts("pep438 [options] <package name> ...", stream=out)
        puts("pep438 [options] -r <requirements file> ...", stream=out)
        puts("cat <requirements file> | pep438 [options]", stream=out)

    puts("\nAdditional Options:")
    with indent(4):
        max_len = max(len(option) for option, _ in options)
        for option, description in options:
            puts(columns([option, max_len + 5], [description, 99]), stream=out)


def process_options(options):
    """Returns given package names or prints help, version, or usage info"""
    packages = []
    for option, arg in options:
        if option in ('-h', '--help'):
            usage()
            sys.exit()
        if option in ('-v', '--version'):
            version()
            sys.exit()
        if option == '-r':
            with open(arg) as f:
                packages += get_pypi_packages(f)
        if option == '-u':
            packages += get_pypi_user_packages(arg)

    return packages


def get_packages_from_stdin():
    """Return packages found from standard input"""
    input_lines = piped_in()
    if input_lines is not None:
        return get_pypi_packages(input_lines)
    else:
        return []


def main():

    packages = []
    args = sys.argv[1:]

    try:
        opts, pkgs = getopt(args, "vhr:u:e", ["version", "help", "requirement",
                                              "user", "errors-only"])
    except GetoptError as e:
        puts(str(e), stream=STDERR)
        usage(error=True)
        sys.exit(2)

    packages += pkgs
    packages += process_options(opts)
    packages += get_packages_from_stdin()
    show_only_errors = ('-e', '') in opts or ('--errors-only', '') in opts

    if not packages:
        # Return error if no packages found
        usage(error=True)
        sys.exit(2)

    for package in packages:
        if valid_package(package):
            urls = get_urls(package)
            symbol = red('\u2717') if urls else green('\u2713')
            msg = "%s %s: %s URLs" % (symbol, blue(package), len(urls))
            if urls or not show_only_errors:
                print(msg)
        else:
            print("%s %s: not found on PyPI" % (red('\u26a0'), blue(package)),
                  file=sys.stderr)



from setuptools import setup, find_packages
import pep438


setup(
    name='pep438',
    version=pep438.__version__,
    author='Trey Hunner',
    url="https://github.com/treyhunner/pep438",
    description="Check your requirements for proper PEP 438 usage.",
    long_description='\n\n'.join((
        open('README.rst').read(),
        open('CHANGES.rst').read(),
    )),
    license=open('LICENSE').read(),
    packages=find_packages(),
    py_modules=['pep438'],
    include_package_data=True,
    install_requires=[
        'requirements-parser==0.0.4',
        'clint',
        'requests',
        'lxml',
    ],
    entry_points={
        'console_scripts': [
            'pep438 = pep438.main:main',
        ],
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
    ],
    tests_require=["mock==1.0.1"],
    test_suite='test_pep438',
)
